#!/usr/bin/env python3

import os
import sys

import metadata

SOLUFILE = os.path.join(metadata.BASEDIR, 'minlplib.solu')

def writesolu(solu = sys.stdout, instances = None, data = None) :
    if data is None :
        data = metadata.read()

    if instances is None :
        instances = list(data.keys())

    for m in sorted(instances) :
        mattribs = data[m]

        if 'removedate' in mattribs :
            continue

        sense = 1.0
        if 'objsense' in mattribs :
            sense = 1.0 if mattribs['objsense'] == 'min' else -1.0

        bestprimal = metadata.getbestprimal(mattribs)
        bestprimal2 = metadata.getbestprimal(mattribs, 1e-6)
        bestdual = metadata.gettrusteddual(mattribs)

        # reduce bestdual to bestprimal2 to accomodate for cases where the optimal value can change a lot with the feasibility tolerance
        if bestdual is not None and abs(bestdual) < float('inf') and bestprimal2 is not None and abs(bestprimal2) < float('inf') :
            if sense * bestprimal2 < sense * bestdual :
                #print >> sys.stderr, m, bestdual, bestprimal2, bestprimal
                bestdual = bestprimal2

        if bestdual is not None and bestdual == sense * float('inf') :
            print('=inf=      ' + m, file=solu)

        elif bestprimal is not None and bestprimal == sense * float('-inf') :
            print('=unbnd=    ' + m, file=solu)

        elif metadata.gap(bestdual, bestprimal) <= metadata.GAPTOL :
            print('=opt=      {0:<30s} {1:30.10f}'.format(m, bestprimal), file=solu)

        else :
            if bestprimal is not None :
                print('=best=     {0:<30s} {1:30.10f}'.format(m, bestprimal), file=solu)
            if bestdual is not None :
                print('=bestdual= {0:<30s} {1:30.10f}'.format(m, bestdual), file=solu)

if __name__ == '__main__' :
    solu = open(SOLUFILE, 'w')
    writesolu(solu)
    solu.close()
